
########################################################
# analyzing factorial experiments
# Copyright (C) 2011, Sascha Hunold <sascha.hunold@imag.fr>
# licensed under GPL v3
########################################################


#########################################################
convert_factorial_results <- function( df ) {

	lambdas <- sort(unique(df$lamb))
	rhos <- sort(unique(df$rho))
	nonrhos <- sort(unique(df$nonrho))
	mus <- sort(unique(df$mu))
	
	df2 <- df[ c("rho","nonrho","mu","lamb","firstiter","relsd","platform","converged")]
	
	df2[ (df2$rho == rhos[1]), ]$rho <- -1
	df2[ (df2$rho == rhos[2]), ]$rho <- +1
	
	df2[ (df2$nonrho == nonrhos[1]), ]$nonrho <- -1
	df2[ (df2$nonrho == nonrhos[2]), ]$nonrho <- +1
	
	df2[ (df2$lamb == lambdas[1]), ]$lamb <- -1
	df2[ (df2$lamb == lambdas[2]), ]$lamb <- +1
	
	df2[ (df2$mu == mus[1]), ]$mu <- -1
	df2[ (df2$mu == mus[2]), ]$mu <- +1
	
	df3 <- df2
	
	df3 <- df3[ with(df3, order(rho, nonrho, mu, lamb, platform)), ]
	
	df3$rho    <- as.factor(df3$rho)
	df3$nonrho <- as.factor(df3$nonrho)
	df3$lamb   <- as.factor(df3$lamb)
	df3$mu     <- as.factor(df3$mu)
	#df3$platform <- as.factor(df3$platform)
	df3$platform <- factor(df3$platform)
	
	return(df3)	
}

#########################################################

get_main_effect <- function(df, val_name, resp_name, low=-1, high=1) {
	eff_low  <- mean(df[ df[c(val_name)]==low, ][c(resp_name)])
    eff_high <- mean(df[ df[c(val_name)]==high, ][c(resp_name)])
	c(eff_low, eff_high)	
}

#########################################################

plot_effect <- function( effects, mean, ylim, main, ylab) {
	plot( c(0,1), effects, type="o", ylab=ylab, main=main, ylim=ylim, xaxt='n', xlab="", xlim=c(-.25,1.25))
	abline(a=mean, b=0)
	axis(side=1, at=c(0,1), label=c("-1", "1"))	
}
#########################################################

plot_main_effects <- function( df, ylim=NULL, mymfrow=NULL ) {

	rhoeff    <- get_main_effect(df, "rho", "relsd")
	nonrhoeff <- get_main_effect(df, "nonrho", "relsd")
	lambeff   <- get_main_effect(df, "lamb", "relsd")
	mueff     <- get_main_effect(df, "mu", "relsd")
	
	if( is.null(ylim) ) {
		ymin <- 0
		ymax <- max( rhoeff, nonrhoeff, lambeff, mueff )
		ylim <- c(ymin, ymax)
	}
	mean <- mean(df$relsd)
	
	if( is.null(mymfrow) ) {
		par(mfrow=c(1, 4))
    } else {
    	par(mfrow=mymfrow,mar=c(2.5,4,2.5,0.5), oma=c(0,0,0,1))
    }
	plot_effect( rhoeff, mean, ylim, 'rho', "cv")
	plot_effect( nonrhoeff, mean, ylim, 'nonrho', "cv")
	plot_effect( mueff, mean, ylim, paste("mu"), "cv")
	plot_effect( lambeff, mean, ylim, 'lamb', "cv")
}

#########################################################
#
# returns a data frame with the anova results
# where the last column contains the significance stars
#
convert_av_res <- function( av_res ) {
	
#   av_res$sig <- c(1:length(av_res[,1]))
#   av_res$sig <- as.character(av_res$sig)
   
   rownames <- rownames(av_res)
   #print(rownames)
   
   df <- data.frame(rownames)
   
   df <- cbind(df, av_res[,1])
   df <- cbind(df, av_res[,2])
   df <- cbind(df, av_res[,3])
   df <- cbind(df, av_res[,4])
   df <- cbind(df, av_res[,5])
   df$sig <- c(1:length(av_res[,1]))
   

   vec <- names(av_res)
   vec <- c("factor", vec)
   vec <- c(vec, "sig")
   names(df) <- vec
   
   cnt <- 1
   for( pval in av_res[,5]) {
   	
   		if( is.na(pval) ) {
   			sig <- " "
   		} else if( pval <= 0.001 ) {
   			sig <- "***"
   		} else if( pval <= 0.01 ) {
   			sig <- "**"
   		} else if( pval <= 0.05 ) {
   			sig <- "*"
   		} else if( pval <= 0.1 ) {
   			sig <- "."
   		} else {
   			sig <- " "
   		}
   		
   		df[cnt,]$sig <- sig
		cnt <- cnt + 1
   }
   
   names(df)[1] <- " "
   names(df)[7] <- " "
   
   return(df)	
}

#####################################
#
# plots boxplot of distribution of firstiter
#####################################


plot_firstiter_dist <- function( df, xlab="", ylab="converging iteration" ) {
	
	boxplot(df$firstiter, xlab=ylab, horizontal=T)	
}



get_overview_table <- function( df ) {
	
	rhos    <- sort(unique(df$rho))
	nonrhos <- sort(unique(df$nonrho))
	mus     <- sort(unique(df$mu))
	lambs   <- sort(unique(df$lamb))

	rnames <- c("low (-1)", "high(1)")
	
	tab <- data.frame(rnames)
	tab <- cbind( tab, rhos )
	tab <- cbind( tab, nonrhos )
	tab <- cbind( tab, mus )
	tab <- cbind( tab, lambs )
	names(tab) <- c(" ", "rho", 'nonrho', "mu", "lamb")
	
	return(tab)
}


analyze_data <- function( file='stats.dat' ) {

########################################

df  <- read.table(file, header=T, sep="\t")
df2 <- convert_factorial_results( df )

##########################################

ov <- get_overview_table( df )
cat('*********************************\n')
cat('Experiment\n')
cat('*********************************\n')
print(ov)
cat('*********************************\n')

##########################################

plot_main_effects( df2, mymfrow=c(2,2) )

##########################################

cat('*********************************\n')
cat('ANOVA: relsd ~ rho * nonrho * lamb * mu + platform\n')
cat('*********************************\n')
av <- aov(relsd ~ rho * nonrho * lamb * mu + platform, data=df2)
av_res <- anova(av)
res <- convert_av_res( av_res )

print(res, include.rownames=F, floating=F)

}



